'use strict';

const express = require('express');
const app = express()
const expressWs = require('express-ws')(app);
const mongoose = require('mongoose');
const hashModel = require('./mongo/hash');
const { Slave } = require('./entity/slave');
const { initSwarmManager, checkInactive } = require('./utils/swarm');
const { slaves } = require('./entity/slave');

const mongoHost = 'URL_MONGO' in process.env ? process.env.URL_MONGO : "mongodb://mongo:27017";
mongoose.connect(mongoHost + '/docker-swarm');

const PORT = 3000



let slaveCount = 0;


/**
 * Insert le hash et la solution dans la base mongo pour les retrouvé plus rapidement 
 * @param hash: le hash
 * @param solution: la solution
 */
function insertHash(hash, solution) {
    let hashMongoose = new hashModel({
        hash: hash,
        solution: solution
    });
    hashMongoose.save(function (err, hashDb) {
        if (err) {
            console.error(err);
        } else {
            console.log("New hash : " + hash + " - " + msg);
        }
    });
}



// Protocol HTML
app.get('/', (req, res) => {
    res.send('Serveur is Up!')
})

// Protocol WS
app.ws('/echo', (ws, req) => {
    ws.on('message', (msg) => {
        console.log(msg)
        ws.send(msg);
    });
});

app.ws('/clients', (ws, req) => {
    ws.on('message', (msg) => {
        const data = msg.split(" ");
        switch (data[0]) {
            case "connect":
                clientsFront.push(ws);
                console.log("Client connected !")
                ws.send("Received")
                break;
            case "updateData":
                sendDataToClients()
                break;
            case "decrypt":
                console.log("Hash received : " + data[1])
                decryptMD5Hash(data[1])
                break;
            case "auto":
                console.log("Auto mode enable")
                console.log("\tspeed : " + data[1])
                autoRun = true
                switch (data[1]) {
                    case "easy":
                        autoDecrypt(60)
                        break;
                    case "medium":
                        autoDecrypt(30)
                        break;
                    case "insane":
                        autoDecrypt(10)
                        break;
                    case "off":
                        console.log("Auto mode disable");
                        autoRun = false;
                        break;
                    default:
                        console.log("\tspeed not available !\nAuto mode disable")
                        autoRun = false
                        break;
                }
                break;
        }
    });
});

app.ws('/slaves', (ws, req) => {
    ws.on('message', (msg) => {
        const data = msg.split(" ");
        switch (data[0]) {
            case "slave":
                slaveCount++;
                slaves.push(
                    new Slave("slave_" + slaveCount, ws)
                );
            case "found":
                sendSolutionToClients(data[1], data[2]);
                insertHash(data[1], data[2]);
                stopSearch(data[1]);
                break;
            default:
                break;
        }
    });
});


initSwarmManager()
const server = app.listen(PORT, () => {
    console.log(`App listening at http://localhost:${PORT}`);
    console.log(`WebSocket listening at ws://localhost:${PORT}`);
});

process.on('SIGINT', () => {
    for (slave in slaves) {
        slave.ws.send("stop");
        slave.ws.send("exit");
    }
    server.close(() => {
        console.log('\nServer closed - All Slaves killed');
        process.exit();
    });
});